import subprocess
import json
import os
import requests as reqs

line_status={
    200: "Success",
    400: "Bad request",
    401: "Invalid access token",
    500: "Failure due to server error"
}

def get_cpu_use():
    snapshot = subprocess.run(["docker", "stats", "--no-stream", "--format", "json"], capture_output=True)
    p_rst = snapshot.stdout.decode("utf-8").strip()

    if p_rst=="":
        print("[INFO] 실행중인 container가 없습니다.")
        return None
    else:
        containers=dict()

        for container in list(map(json.loads,snapshot.stdout.decode("utf-8").strip().split("\n"))):
            containers[container["Name"]]=container["CPUPerc"]

        cnts = len(list(filter(lambda x:"samdul-blog" in x,containers.keys())))

        if cnts==0:
            print("[INFO] 실행중인 container가 없습니다.")

            return None,None

        cpu_use = containers["samdul-blog-1"]

        return cpu_use, cnts

def get_config():
    from configparser import ConfigParser

    cfp = ConfigParser()

    cfp.read(f"{os.path.dirname(os.path.abspath(__file__))}/config/config.ini")

    return cfp

def get_limit():

    limit_cfg = get_config()["limit"]

    return limit_cfg["scale_in_value"],limit_cfg["scale_out_value"]

def send_line_noti(message):
    # curl -X POST -H "Authorization: Bearer $LINE_TOKEN" -F "message=1"
    LINE_TOKEN=os.getenv("LINE_TOKEN")
    if LINE_TOKEN==None:
        print("[WARN] LINE_TOKEN이 존재하지 않습니다.")
    else:
        headers={
            "Authorization" : f"Bearer {LINE_TOKEN}"
        }
        msg={
            "message" : message
        }

        status=reqs.post("https://notify-api.line.me/api/notify", data=msg, headers=headers).status_code

        return (status,line_status[status]) if status in line_status else (status,"Processed over time or stopped")



from auto_scailer.utils import get_limit, get_cpu_use, send_line_noti
import time
import os

conti_high=0
conti_low=0

def auto_scailer():
    global conti_high
    global conti_low

    scale_in_value, scale_out_value=map(float,get_limit())

    home_path=os.path.expanduser("~")
    cu, scale_cnt =get_cpu_use()
    while scale_cnt:
        print(f"[INFO] 현재 CPU사용량은 {cu}입니다.")
        print(f"[INFO] 현재 container의 수는 {scale_cnt}개입니다.")
        #### CPU 사용량이 scale_out_value를 넘으면 scale out ####
        if float(cu.replace("%",""))>scale_out_value:
            conti_high+=10
            print(f"[WARN] {conti_high}초 동안 과부하 상태...")
        else:
            conti_high=0

        if conti_high==60:
            print(f"[INFO] container의 수를 {scale_cnt+1}로 scale out 합니다.")
            os.system(f"docker compose -f {home_path}/code/docker/k1s/docker-compose.yaml up -d --scale blog={scale_cnt+1}")

            conti_high=0
            code, msg = send_line_noti(f"[INFO] container의 수가 {scale_cnt+1}로 scale out 되었습니다.")


        ######################################################
        #### CPU 사용량이 scale_in_value보다 낮으면 scale in ####
        ##### 1개의 컨테이너는 남겨야 함 ########################
        if scale_cnt>1:
            if float(cu.replace("%",""))<scale_in_value:
                conti_low+=10
                print(f"[INFO] {conti_low}초 동안 안정된 상태...")
            else:
                conti_low=0

            if conti_low==60:
                    print(f"[INFO] container의 수를 {scale_cnt-1}로 scale in 합니다.", end="\n\n")
                    os.system(f"docker compose -f {home_path}/code/docker/k1s/docker-compose.yaml up -d --scale blog={scale_cnt-1}")

                    conti_low=0
                    code, msg = send_line_noti(f"[INFO] container의 수가 {scale_cnt-1}로 scale in 되었습니다.")
        ######################################################
        print("")
        time.sleep(10)
        cu, scale_cnt =get_cpu_use()